﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using emanual.Wpf.TextEditor; // g

namespace AvalonEditTest
{
	public partial class Customize : Window
	{
		private string FText = String.Empty;
		private List<CustomColor> FColorList;
		public Window OwnerWindow = null;

		private emanual.Wpf.TextEditor.SearchManager FSearchManager;
		private ICSharpCode.AvalonEdit.Search.SearchResultBackgroundRenderer FSearchRenderer = null;

		//---------------------------------------------------------------------------------------------
		public Customize()
		{
			InitializeComponent();
		}

		//---------------------------------------------------------------------------------------------
		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			if (this.OwnerWindow != null)
			{
				this.Left = this.OwnerWindow.Left + 50;
				this.Top = this.OwnerWindow.Top + 50;
			}
			else
				this.WindowStartupLocation = WindowStartupLocation.CenterOwner;

			//------------------------------------------------------
			var ini = new emanual.Wpf.Utility.XmlIni(g.INI_FILE);

			string fontName = (string)ini.ReadValue(g.GENERAL_SEC, g.FONTFAMILY_KEY, "MS Gothic");
			var fontFamily = new FontFamily(fontName);

			foreach (FontFamily family in Fonts.SystemFontFamilies)
			{
				cmbFontFamily.Items.Add(family.Source);
			}

			// 昇順でソートする
			cmbFontFamily.Items.SortDescriptions.Add(new System.ComponentModel.SortDescription(null, System.ComponentModel.ListSortDirection.Ascending));

			for (int i = 0; i < cmbFontFamily.Items.Count; ++i)
			{
				var family = new FontFamily((string)cmbFontFamily.Items[i]);

				if (family.Equals(fontFamily) == true)
				{
					cmbFontFamily.SelectedIndex = i;
					break;
				}
			}

			//-------------------------------------------------------
			for (int i = 10; i <= 20; ++i)
			{
				cmbFontSize.Items.Add(i);
			}

			int fontSize = (int)ini.ReadValue(g.GENERAL_SEC, g.FONTSIZE_KEY, 13);
			cmbFontSize.SelectedIndex = fontSize - 10;

			//-------------------------------------------------------
			int tabSize = (int)ini.ReadValue(g.GENERAL_SEC, g.TABSIZE_KEY, 2);

			if (tabSize == 2)
				rdTab2.IsChecked = true;
			else if (tabSize == 4)
				rdTab4.IsChecked = true;
			else
				rdTab8.IsChecked = true;

			//-------------------------------------------------------
			string newLine = (string)ini.ReadValue(g.GENERAL_SEC, g.NEWLINE_KEY, "\u21B5");

			if (String.Compare((string)rdNewLine1.Tag, newLine, true) == 0)
				rdNewLine1.IsChecked = true;
			else if (String.Compare((string)rdNewLine2.Tag, newLine, true) == 0)
				rdNewLine2.IsChecked = true;
			else
				rdNewLine3.IsChecked = true;

			//-------------------------------------------------------
			SolidColorBrush brush = null;
			FColorList = new List<CustomColor>();
			brush = (SolidColorBrush)ini.ReadValue(g.BRUSH_SEC, g.BACKGROUND_KEY, SystemColors.WindowBrush);
			FColorList.Add(new CustomColor("背景色", brush, 0));
			brush = (SolidColorBrush)ini.ReadValue(g.BRUSH_SEC, g.FOREGROUND_KEY, SystemColors.WindowTextBrush);
			FColorList.Add(new CustomColor("文字色", brush, 1));
			brush = (SolidColorBrush)ini.ReadValue(g.BRUSH_SEC, g.CARET_KEY, SystemColors.WindowTextBrush);
			FColorList.Add(new CustomColor("キャレット", brush, 2));
			brush = (SolidColorBrush)ini.ReadValue(g.BRUSH_SEC, g.HYPERLINK_KEY, Brushes.Blue);
			FColorList.Add(new CustomColor("メールアドレスおよび URI", brush, 3));
			brush = (SolidColorBrush)ini.ReadValue(g.BRUSH_SEC, g.LINENUMBER_KEY, Brushes.Gray);
			FColorList.Add(new CustomColor("行番号", brush, 4));
			brush = (SolidColorBrush)ini.ReadValue(g.BRUSH_SEC, g.NEWLINE_KEY, Brushes.DimGray);
			FColorList.Add(new CustomColor("改行文字、スペース、タブ", brush, 5));
			brush = (SolidColorBrush)ini.ReadValue(g.BRUSH_SEC, g.SEARCH_KEY, Brushes.Orange);
			FColorList.Add(new CustomColor("検索位置の強調表示", brush, 6));

			lstColor.ItemsSource = FColorList;

			//-------------------------------------------------------

			// TextEditor の色の設定
			textEditor.Background = FColorList[0].Brush;
			textEditor.Foreground = FColorList[1].Brush;
			textEditor.TextArea.Caret.CaretBrush = FColorList[2].Brush;
			textEditor.TextArea.TextView.LinkTextForegroundBrush = FColorList[3].Brush;
			textEditor.LineNumbersForeground = FColorList[4].Brush;
			textEditor.TextArea.TextView.NonPrintableCharacterBrush = FColorList[5].Brush;
			textEditor.Options.SearchMarkerBrush = FColorList[6].Brush;

			// TextEditor のその他の設定
			textEditor.Options.ShowEndOfLine = true;
			textEditor.Options.ShowSpaces = true;
			textEditor.Options.ShowTabs = true;
			textEditor.WordWrap = true;
			textEditor.ShowLineNumbers = true;
			textEditor.Encoding = System.Text.Encoding.UTF8;
			textEditor.TextArea.TextView.Margin = new Thickness(4, 2, 0, 0); // 上辺と左辺を少しあける

			textEditor.Options.ConvertTabsToSpaces = true;
			textEditor.Options.IndentationSize = (int)ini.ReadValue(g.GENERAL_SEC, g.TABSIZE_KEY, 2);
			textEditor.Options.InheritWordWrapIndentation = false;
			textEditor.Options.CutCopyWholeLine = false;
			textEditor.Options.NewLine = newLine;

			FText = "これは色見本用テキストです。\r\n\r\n" + 
					"メールアドレス：pmansato@kanazawa-net.ne.jp\r\n" +
					"    スペースをあらわす文字の例で、左側に 4 つのスペースがあります。\r\n" +
					"\t\tタブをあらわす文字の例で、左側に 2 つのタブがあります。タブのサイズもチェックしてください。\r\n" +
					"検索位置を強調表示する例で、「強調」を強調しています。\r\n" +
					"改行文字やその文字色および行番号の色も確認してください。";

			textEditor.Text = FText;

			FSearchManager = new emanual.Wpf.TextEditor.SearchManager(textEditor, FSearchRenderer);
			FSearchRenderer = FSearchManager.SearchAll(textEditor, "強調", true, false, ICSharpCode.AvalonEdit.Search.SearchMode.Normal);
		}

		//---------------------------------------------------------------------------------------------
		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			FSearchManager.Close(textEditor, FSearchRenderer);
			FSearchRenderer = null;
		}

		//---------------------------------------------------------------------------------------------
		private void btnOK_Click(object sender, RoutedEventArgs e)
		{
			var ini = new emanual.Wpf.Utility.XmlIni(g.INI_FILE);

			ini.WriteValue(g.GENERAL_SEC, g.FONTFAMILY_KEY, cmbFontFamily.Text);
			ini.WriteValue(g.GENERAL_SEC, g.FONTSIZE_KEY, cmbFontSize.Text);

			string newLine = (string)rdNewLine2.Tag;
			if (rdNewLine1.IsChecked == true)
				newLine = (string)rdNewLine1.Tag;
			else if (rdNewLine3.IsChecked == true)
				newLine = (string)rdNewLine3.Tag;

			ini.WriteValue(g.GENERAL_SEC, g.NEWLINE_KEY, newLine);

			int tabSize = 2;
			if (rdTab4.IsChecked == true)
				tabSize = 4;
			else if (rdTab8.IsChecked == true)
				tabSize = 8;

			ini.WriteValue(g.GENERAL_SEC, g.TABSIZE_KEY, tabSize);

			//---------------------------------------------------------
			ini.WriteValue(g.BRUSH_SEC, g.BACKGROUND_KEY, FColorList[0].Brush);
			ini.WriteValue(g.BRUSH_SEC, g.FOREGROUND_KEY, FColorList[1].Brush);
			ini.WriteValue(g.BRUSH_SEC, g.CARET_KEY, FColorList[2].Brush);
			ini.WriteValue(g.BRUSH_SEC, g.HYPERLINK_KEY, FColorList[3].Brush);
			ini.WriteValue(g.BRUSH_SEC, g.LINENUMBER_KEY, FColorList[4].Brush);
			ini.WriteValue(g.BRUSH_SEC, g.NEWLINE_KEY, FColorList[5].Brush);
			ini.WriteValue(g.BRUSH_SEC, g.SEARCH_KEY, FColorList[6].Brush);

			this.DialogResult = true;
		}

		//---------------------------------------------------------------------------------------------
		private void btnCancel_Click(object sender, RoutedEventArgs e)
		{
			this.DialogResult = false;
		}

		//---------------------------------------------------------------------------------------------
		private void btnColor_Click(object sender, RoutedEventArgs e)
		{
			Button button = sender as Button;

			if (button != null)
			{
				int tag = (int)button.Tag;

				var dlg = new emanual.Wpf.Controls.ColorPicker();
				dlg.OwnerWindow = this; // ダイアログの表示位置の基準を現在のウインドウにする

				if (dlg.ShowDialog() == true)
				{
					var brush = new SolidColorBrush(dlg.SelectedColor);
					FColorList[tag] = new CustomColor(FColorList[tag].Part, brush, tag);

					lstColor.ItemsSource = null;
					lstColor.ItemsSource = FColorList;

					this.UpdateControlColor(tag);
				}
			}
		}

		//---------------------------------------------------------------------------------------------
		private void cmbFontFamily_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			if (String.IsNullOrEmpty(cmbFontFamily.Text) == true)
				return;

			this.BeginUpdate();
			textEditor.FontFamily = new FontFamily(cmbFontFamily.Text);
			this.EndUpdate();
		}

		//---------------------------------------------------------------------------------------------
		private void cmbFontSize_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			if (String.IsNullOrEmpty(cmbFontSize.Text) == true)
				return;

			this.BeginUpdate();
			textEditor.FontSize = Convert.ToDouble(cmbFontSize.Text);
			this.EndUpdate();
		}

		//---------------------------------------------------------------------------------------------
		private void rdNewLine_Checked(object sender, RoutedEventArgs e)
		{
			this.BeginUpdate();

			if (rdNewLine1.IsChecked == true)
				textEditor.Options.NewLine = "¶";
			else if (rdNewLine2.IsChecked == true)
				textEditor.Options.NewLine = "↵";
			else
				textEditor.Options.NewLine = "↓";

			this.EndUpdate();
		}

		//---------------------------------------------------------------------------------------------
		private void rdTabSize_Checked(object sender, RoutedEventArgs e)
		{
			this.BeginUpdate();

			if (rdTab2.IsChecked == true)
				textEditor.Options.IndentationSize = 2;
			else if (rdTab4.IsChecked == true)
				textEditor.Options.IndentationSize = 4;
			else
				textEditor.Options.IndentationSize = 8;

			this.EndUpdate();
		}

		//---------------------------------------------------------------------------------------------
		private void BeginUpdate()
		{
			if (textEditor.Document == null)
				return;

			textEditor.Document.Text = String.Empty;
		}

		//---------------------------------------------------------------------------------------------
		private void EndUpdate()
		{
			if (textEditor.Document == null)
				return;

			textEditor.Document.Text = FText;

			if (FSearchRenderer != null)
				FSearchManager.SearchAll(textEditor, "強調");
		}

		//---------------------------------------------------------------------------------------------
		private void UpdateControlColor(int tag)
		{
			this.BeginUpdate();
			var brush = FColorList[tag].Brush;

			switch (tag)
			{
				case 0:
					textEditor.Background = brush;
					break;
				case 1:
					textEditor.Foreground = brush;
					break;
				case 2:
					textEditor.TextArea.Caret.CaretBrush = brush;
					break;
				case 3:
					textEditor.TextArea.TextView.LinkTextForegroundBrush = brush;
					break;
				case 4:
					textEditor.LineNumbersForeground = brush;
					break;
				case 5:
					textEditor.TextArea.TextView.NonPrintableCharacterBrush = brush;
					break;
				case 6:
					textEditor.Options.SearchMarkerBrush = brush;
					break;
			}

			this.EndUpdate();
		}
	} // end of Customize class

	//***********************************************************************************************
	internal class CustomColor
	{
		public string Part { get; set; }
		public SolidColorBrush Brush { get; set; }
		public int Tag { get; set; }

		public CustomColor(string part, SolidColorBrush brush, int tag)
		{
			this.Part = part;
			this.Brush = brush;
			this.Tag = tag;
		}
	} // end of CustomColor class
} // end of AvalonEditTest namespace
