﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Input; // Key
using System.IO;
using System.Windows.Threading; // DispatcherTimer
using ICSharpCode.AvalonEdit.Folding; // FoldingManager
using ICSharpCode.AvalonEdit.Highlighting; // HighlightingManage
using Microsoft.Win32; // OpenFileDialog
using System.Diagnostics;
using emanual.Wpf.TextEditor;
using ICSharpCode.AvalonEdit.Document;
using System.Windows.Media.TextFormatting; // TextLine
using ICSharpCode.AvalonEdit.Rendering; // VisualLine
using ICSharpCode.AvalonEdit.Highlighting.Xshd;
using ICSharpCode.AvalonEdit.Search;
using ICSharpCode.AvalonEdit.Utils; // Rope
using System.Linq;

namespace AvalonEditTest
{
	public partial class MainWindow : Window
	{
		private const string DEFAULT_FILENAME = "無題.txt";

		private string FCurrentFileName = DEFAULT_FILENAME;
		private FoldingManager FFoldingManager = null;
		private AbstractFoldingStrategy FFoldingStrategy = null;
		private IHighlightingDefinition FCurrentDefinition = null;
		private IHighlightingDefinition FTxtDefinition = null; // TXT 用ハイライト定義
		private emanual.Wpf.Utility.HistoryList FSearchList;

		private emanual.Wpf.TextEditor.SearchManager FSearchManager;
		private SearchResultBackgroundRenderer FSearchRenderer = null;

		// プロパティ
		public SearchManager SearchManager { get { return FSearchManager; } set { FSearchManager = value; } }
		public SearchResultBackgroundRenderer SearchRenderer { get { return FSearchRenderer; } set { FSearchRenderer = value; } }

		//---------------------------------------------------------------------------------------------
		public MainWindow()
		{
			InitializeComponent();
		}

		//---------------------------------------------------------------------------------------------
		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			var xmlIni = new emanual.Wpf.Utility.XmlIni(g.INI_FILE);

			// 初期化ファイルに記録した前回起動時のフォームの位置とサイズを再現する
			xmlIni.SetFormPositionAndSize(this, g.GENERAL_SEC, g.MAIN_FORM_POS_KEY, 100, 100, 600, 400);

			mnuShowFunctionBar.IsChecked = (bool)xmlIni.ReadValue(g.GENERAL_SEC, g.FUNCTION_BAR_KEY, true);

			if (mnuShowFunctionBar.IsChecked)
				borderFunction.Visibility = Visibility.Visible;
			else
				borderFunction.Visibility = Visibility.Collapsed;

			// 初期化ファイルから過去の項目を読み込み、cmbSearch に設定する
			this.SetupSearchComboBox();

			// 初期化ファイルからデータを読み込んで、TextEditor コントロールを初期化する
			this.InitializeControl();

			// このテスト用アプリケーションとしての TextEditor コントロールの設定
			textEditor.WordWrap = true;
			textEditor.ShowLineNumbers = false;
			textEditor.Encoding = System.Text.Encoding.UTF8;

			textEditor.TextArea.Caret.PositionChanged += Caret_PositionChanged;
			textEditor.TextArea.SelectionCornerRadius = 0; // オリジナルのデフォルトは 3
			textEditor.TextArea.TextView.Margin = new Thickness(4, 2, 0, 0); // 上辺と左辺を少しあける

			textEditor.Options.ConvertTabsToSpaces = true;
			textEditor.Options.InheritWordWrapIndentation = false;
			textEditor.Options.CutCopyWholeLine = false;
			textEditor.Options.ShowEndOfLine = true;
			textEditor.Options.MarkerCornerRadius = 0; // 独自の機能として追加したもの（オリジナル版では 3）

			// .txt 用ハイライト表示
			FTxtDefinition = HighlightingManager.Instance.GetDefinitionByExtension(".txt");
			textEditor.SyntaxHighlighting = FTxtDefinition;

			// 検索マネージャ
			FSearchManager = new emanual.Wpf.TextEditor.SearchManager(textEditor, FSearchRenderer);

			statusLabel1.Text = FCurrentFileName;
			statusLabel3.Text = textEditor.Encoding.WebName;

			// IME
			InputMethod.Current.StateChanged += new InputMethodStateChangedEventHandler(inputMethod_StateChanged);

			textEditor.Text = "メニュの [ファイル]-[開く] を使って、サンプルテキストの Sample.txt を選択してください。";
		}

		//---------------------------------------------------------------------------------------------
		private void inputMethod_StateChanged(object sender, InputMethodStateChangedEventArgs e)
		{
			statusLabel4.Text = InputMethod.Current.ImeState.ToString();
		}

		//---------------------------------------------------------------------------------------------
		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			var xmlIni = new emanual.Wpf.Utility.XmlIni(g.INI_FILE);

			// 現在のフォームの位置とサイズを初期化ファイルに記録する
			xmlIni.WriteFormPositionAndSize(this, g.GENERAL_SEC, g.MAIN_FORM_POS_KEY);

			// ファンクションバーの表示・非表示を初期化ファイルに記録する
			xmlIni.WriteValue(g.GENERAL_SEC, g.FUNCTION_BAR_KEY, mnuShowFunctionBar.IsChecked);

			if (FSearchRenderer != null)
			{
				FSearchManager.Close(textEditor, FSearchRenderer);
			}

			// 検索コンボボックスの内容を初期化ファイルに出力する
			this.WriteSearchComboBox();

			InputMethod.Current.StateChanged -= new InputMethodStateChangedEventHandler(inputMethod_StateChanged);
		}

		//---------------------------------------------------------------------------------------------
		private void Window_PreviewKeyDown(object sender, KeyEventArgs e)
		{
			switch (e.Key)
			{
				case Key.System:
					if (e.SystemKey == Key.F10) // [F10] は Windows が使うので、それを横取りする
					{
						this.funcButton_Click(btnF10, null);
						e.Handled = true;
					}
					break;
				case Key.F1:
					this.funcButton_Click(btnF1, null);
					break;
				case Key.F2:
					this.funcButton_Click(btnF2, null);
					break;
				case Key.F3:
					this.funcButton_Click(btnF3, null);
					break;
				case Key.F4:
					this.funcButton_Click(btnF4, null);
					break;
				case Key.F5:
					this.funcButton_Click(btnF5, null);
					break;
				case Key.F6:
					this.funcButton_Click(btnF6, null);
					break;
				case Key.F7:
					this.funcButton_Click(btnF7, null);
					break;
				case Key.F8:
					this.funcButton_Click(btnF8, null);
					break;
				case Key.F9:
					this.funcButton_Click(btnF9, null);
					break;
				case Key.F11:
					this.funcButton_Click(btnF11, null);
					break;
				case Key.F12:
					this.funcButton_Click(btnF12, null);
					break;
				case Key.Escape:
					if (FSearchRenderer != null)
					{
						FSearchManager.Close(textEditor, FSearchRenderer);
						FSearchRenderer = null;
					}
					break;
				case Key.A:
					if (Keyboard.Modifiers == ModifierKeys.Control)
						this.funcButton_Click(btnF12, null);
					break;

				case Key.Home: // WordWrap = true 以外のときは既定の動作
					if ((Keyboard.Modifiers == ModifierKeys.None) && (textEditor.WordWrap == true))
					{
						TextEditorUtility.MoveCaretToStartOfLine(textEditor);
						e.Handled = true; // 既定の動作をキャンセルする
					}
					break;

				case Key.End: // WordWrap = true 以外のときは既定の動作
					if ((Keyboard.Modifiers == ModifierKeys.None) && (textEditor.WordWrap == true))
					{
						TextEditorUtility.MoveCaretToEndOfLine(textEditor);
						e.Handled = true; // 既定の動作をキャンセルする
					}
					break;
			}
		}

		#region mnuFile
		//---------------------------------------------------------------------------------------------
		private void mnuFile_Click(object sender, RoutedEventArgs e)
		{
			string name = this.GetSelectedItemName(sender);

			switch (name)
			{
				case "mnuFileNew":
					FCurrentFileName = DEFAULT_FILENAME;
					statusLabel1.Text = FCurrentFileName;
					textEditor.Text = String.Empty;
					textEditor.SyntaxHighlighting = FTxtDefinition;
					textEditor.Encoding = System.Text.Encoding.UTF8;
					textEditor.WordWrap = true;
					statusLabel3.Text = textEditor.Encoding.WebName;
					break;

				case "mnuFileOpen":
				case "btnFileOpen":
					var openDlg = new OpenFileDialog();
					openDlg.Filter = "テキスト (*.txt)|*.txt|C# コード(*.cs)|*.cs|HTML ファイル(*.htm)|*.htm|XML ファイル(*.xml)|*.xml|すべてのファイル(*.*)|*.*";
					openDlg.CheckFileExists = true;
					openDlg.InitialDirectory = TextEditorUtility.GetExecutableFolder();

					if (openDlg.ShowDialog() == true)
					{
						FCurrentFileName = openDlg.FileName;
						textEditor.Load(FCurrentFileName);

						// statusLabel1 にファイル名を表示する（必要なら枠内に納まるように短縮する）
						string fileName = TextEditorUtility.GetEllipsisFileName(statusBarItem1.ActualWidth, FCurrentFileName, this.FontSize,
								new Typeface(this.FontFamily, this.FontStyle, this.FontWeight, this.FontStretch), 0);
						statusLabel1.Text = fileName;

						if (Path.GetExtension(FCurrentFileName) != ".txt") // テキストファイル以外のときは組み込み済みのハイライトを有効にする
						{
							FCurrentDefinition = HighlightingManager.Instance.GetDefinitionByExtension(Path.GetExtension(FCurrentFileName));
							textEditor.SyntaxHighlighting = FCurrentDefinition;
						}
						else // TXT ファイルのとき
						{
							textEditor.SyntaxHighlighting = FTxtDefinition;

							if (FFoldingStrategy != null)
							{
								if (FFoldingManager != null)
								{
									FoldingManager.Uninstall(FFoldingManager);
									FFoldingManager = null;
									FFoldingStrategy = null;
								}
							}
						}

						// ロードするドキュメントの種類に基づいてインデント方法を設定する
						switch (textEditor.SyntaxHighlighting.Name)
						{
							case "XML":
								textEditor.TextArea.IndentationStrategy = new ICSharpCode.AvalonEdit.Indentation.DefaultIndentationStrategy();
								break;
							case "C#":
							case "C++":
							case "PHP":
							case "Java":
								textEditor.TextArea.IndentationStrategy = new ICSharpCode.AvalonEdit.Indentation.CSharp.CSharpIndentationStrategy(textEditor.Options);
								break;
							default:
								textEditor.TextArea.IndentationStrategy = new ICSharpCode.AvalonEdit.Indentation.DefaultIndentationStrategy();
								break;
						}

						// キャレットをドキュメントの先頭に移動
						TextEditorUtility.MoveToDocumentStart(textEditor);

						// ドキュメントのエンコーディング法を statusLabel3 に表示する
						statusLabel3.Text = textEditor.Encoding.WebName;
					}

					break;

				case "mnuFileSave":
				case "btnFileSave":
					textEditor.Save(FCurrentFileName);
					break;

				case "mnuFileSaveAs":
					var saveDlg = new SaveFileDialog();
					saveDlg.Filter = "テキスト (*.txt)|*.txt|C# コード(*.cs)|*.cs|HTML ファイル(*.htm)|*.htm|すべてのファイル(*.*)|*.*";
					saveDlg.DefaultExt = ".txt";
					saveDlg.FileName = Path.GetFileName(FCurrentFileName);
					saveDlg.InitialDirectory = TextEditorUtility.GetExecutableFolder();

					if (saveDlg.ShowDialog() == true)
						FCurrentFileName = saveDlg.FileName;
					else
						return;

					textEditor.Save(FCurrentFileName);
					break;

				case "mnuFileSaveHtml":
					if (FCurrentDefinition == null)
					{
						MessageBox.Show("ハイライト表示が無効です。");
						return;
					}

					string htmlFileName;
					var htmlSaveDlg = new SaveFileDialog();
					htmlSaveDlg.FileName = Path.GetFileNameWithoutExtension(FCurrentFileName) + ".htm";
					htmlSaveDlg.DefaultExt = ".htm";

					if (htmlSaveDlg.ShowDialog() == true)
						htmlFileName = htmlSaveDlg.FileName;
					else
						return;

					string html = TextEditorUtility.GetHtmlDocument(textEditor, FCurrentDefinition);

					var htmlWriter = new StreamWriter(htmlFileName);
					htmlWriter.WriteLine(html);
					htmlWriter.Close();

					break;

				case "mnuFileSaveFlow":
					var flowDocument = TextEditorUtility.GetFlowDocument(textEditor);
					var range = new System.Windows.Documents.TextRange(flowDocument.ContentStart, flowDocument.ContentEnd);

					string flowFileName;
					var flowSaveDlg = new SaveFileDialog();
					flowSaveDlg.FileName = Path.GetFileNameWithoutExtension(FCurrentFileName) + ".flowpackage";
					flowSaveDlg.DefaultExt = ".flowpackage";

					if (flowSaveDlg.ShowDialog() == true)
						flowFileName = flowSaveDlg.FileName;
					else
						return;

					Stream stream = null;

					try
					{
						stream = File.Create(flowFileName);
						range.Save(stream, DataFormats.XamlPackage);
					}
					catch (Exception ex)
					{
						MessageBox.Show(ex.Message);
					}
					finally
					{
						stream.Close();
					}

					break;

				case "mnuFileSaveFlowXaml":
					var flowDocument2 = TextEditorUtility.GetFlowDocument(textEditor);
					var range2 = new System.Windows.Documents.TextRange(flowDocument2.ContentStart, flowDocument2.ContentEnd);

					string flowFileName2;
					var flowSaveDlg2 = new SaveFileDialog();
					flowSaveDlg2.FileName = Path.GetFileNameWithoutExtension(FCurrentFileName) + ".xaml";
					flowSaveDlg2.DefaultExt = ".xaml";

					if (flowSaveDlg2.ShowDialog() == true)
						flowFileName2 = flowSaveDlg2.FileName;
					else
						return;

					Stream stream2 = null;

					try
					{
						stream2 = File.Create(flowFileName2);
						range2.Save(stream2, DataFormats.Xaml);
					}
					catch (Exception ex)
					{
						MessageBox.Show(ex.Message);
					}
					finally
					{
						stream2.Close();
					}
					break;

				case "mnuFilePrint":
					MessageBox.Show("実装はありません。");
					break;

				case "mnuFileQuit":
					this.Close();
					break;
			}
		}
		#endregion

		#region mnuEdit
		//---------------------------------------------------------------------------------------------
		private void mnuEdit_Click(object sender, RoutedEventArgs e)
		{
			string name = this.GetSelectedItemName(sender);

			switch (name)
			{
				case "mnuEditDeleteLeadingWhitespace":
					TextEditorUtility.DeleteLeadingWhitespace(textEditor);
					break;
				case "mnuEditDeleteTrailingWhitespace":
					TextEditorUtility.DeleteTrailingWhitespace(textEditor);
					break;
				case "mnuEditSortAscend":
					TextEditorUtility.SortSelectedSection(textEditor);
					break;
				case "mnuEditSortDescend":
					TextEditorUtility.SortSelectedSection(textEditor, false);
					break;
				case "mnuEditDeleteEmptyLine":
					TextEditorUtility.DeleteEmptyLine(textEditor);
					break;
				case "mnuEditUniqueLine":
					TextEditorUtility.UniqueLine(textEditor);
					break;
				case "mnuEditWordSpace":
					TextEditorUtility.InsertWordSpace(textEditor);
					break;
				case "mnuEditIndent":
					TextEditorUtility.IndentSelectedSection(textEditor);
					break;
				case "mnuEditRemoveIndent":
					TextEditorUtility.UnindentSelectedSection(textEditor);
					break;
				case "mnuEditInsertImage":
					textEditor.TextArea.TextView.ElementGenerators.Add(new ImageElementGenerator(Directory.GetCurrentDirectory()));
					break;
			}
		}
		#endregion

		#region mnuEditCommand
		//---------------------------------------------------------------------------------------------
		private void mnuEditCommand_Click(object sender, RoutedEventArgs e)
		{
			string name = this.GetSelectedItemName(sender);

			switch (name)
			{
				case "mnuEditLeadingSpacesToTabs":
					ICSharpCode.AvalonEdit.AvalonEditCommands.ConvertLeadingSpacesToTabs.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditLeadingTabsToSpaces":
					ICSharpCode.AvalonEdit.AvalonEditCommands.ConvertLeadingTabsToSpaces.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditSpacesToTabs":
					ICSharpCode.AvalonEdit.AvalonEditCommands.ConvertSpacesToTabs.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditTabsToSpaces":
					ICSharpCode.AvalonEdit.AvalonEditCommands.ConvertTabsToSpaces.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditToLowercase":
					ICSharpCode.AvalonEdit.AvalonEditCommands.ConvertToLowercase.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditToTitlecase":
					ICSharpCode.AvalonEdit.AvalonEditCommands.ConvertToTitleCase.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditToUppercase":
					ICSharpCode.AvalonEdit.AvalonEditCommands.ConvertToUppercase.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditDeleteLine":
					ICSharpCode.AvalonEdit.AvalonEditCommands.DeleteLine.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditIndentSelection":
					ICSharpCode.AvalonEdit.AvalonEditCommands.IndentSelection.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditInvertCase":
					ICSharpCode.AvalonEdit.AvalonEditCommands.InvertCase.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditRemoveLeadingSpaces":
					ICSharpCode.AvalonEdit.AvalonEditCommands.RemoveLeadingWhitespace.Execute(null, textEditor.TextArea);
					break;
				case "mnuEditRemoveTrailingSpaces":
					ICSharpCode.AvalonEdit.AvalonEditCommands.RemoveTrailingWhitespace.Execute(null, textEditor.TextArea);
					break;
			}
		}
		#endregion

		#region mnuSearch
		//---------------------------------------------------------------------------------------------
		private void mnuSearch_Click(object sender, RoutedEventArgs e)
		{
			string name = this.GetSelectedItemName(sender);

			switch (name)
			{
				case "mnuSearch":
				case "btnSearch":
					if (FSearchRenderer != null)
						FSearchManager.Close(textEditor, FSearchRenderer);

					var searchForm = new emanual.Wpf.TextEditor.TextSearch(this, textEditor);
					// キャレット位置の語句を切り出し、検索文字列の初期値とする
					searchForm.SearchString = TextEditorUtility.GetWordFromCaretPosition(textEditor.Document.Text, textEditor.TextArea.Caret.Offset);

					if (searchForm.ShowDialog() == true)
					{
						FSearchRenderer = FSearchManager.SearchAll(textEditor, searchForm.SearchString, searchForm.IgnoreCase, searchForm.IsWholeWord, searchForm.SearchMode);
					}
					break;

				case "mnuSearchNext":
					FSearchManager.SearchNext();
					break;

				case "mnuSearchPrevious":
					FSearchManager.SearchPrevious();
					break;

				case "btnSearchAll":
					textEditor.TextArea.Caret.Offset = 0;
					FSearchList.AddNew(cmbSearch.Text);
					//cmbSearch.ItemsSource = null;
					cmbSearch.ItemsSource = FSearchList;

					if (FSearchRenderer != null)
					{
						FSearchManager.Close(textEditor, FSearchRenderer);
						FSearchRenderer = null;
					}
					else
						FSearchRenderer = FSearchManager.SearchAll(textEditor, cmbSearch.Text);

					break;

				case "mnuReplace":
				case "btnReplace":
					var replaceForm = new emanual.Wpf.TextEditor.TextReplace(this, textEditor);
					// キャレット位置の語句を切り出す
					replaceForm.SearchString = TextEditorUtility.GetWordFromCaretPosition(textEditor.Document.Text, textEditor.TextArea.Caret.Offset);

					if (replaceForm.ShowDialog() == true)
					{
						FSearchRenderer = FSearchManager.SearchAll(textEditor, replaceForm.SearchString, replaceForm.IgnoreCase, replaceForm.IsWholeWord, replaceForm.SearchMode);
						FSearchManager.SearchString = replaceForm.SearchString;
						FSearchManager.ReplaceString = replaceForm.ReplaceString;

						if (replaceForm.IsReplaceAll)
						{
							FSearchManager.ReplaceAll();
							FSearchManager.Close(textEditor, FSearchRenderer);
							FSearchRenderer = null;
						}
					}
					break;

				case "btnDoReplace": // フォーカスのある位置を置換する
					if ((FSearchRenderer != null) && (FSearchRenderer.CurrentResults.Count > 0))
					{
						FSearchManager.Replace();

						// 置換を実行したことで SearchResult のデータがずれることがあるので、現在の検索状態を解除し、あらためて検索する
						FSearchManager.CancelSearchAll(textEditor, FSearchRenderer);
						FSearchRenderer = FSearchManager.SearchAll(textEditor, FSearchManager.SearchString);
					}

					break;
			}
		}
		#endregion

		#region mnuShow
		//---------------------------------------------------------------------------------------------
		private void mnuShow_Click(object sender, RoutedEventArgs e)
		{
			string name = this.GetSelectedItemName(sender);

			switch (name)
			{
				case "mnuShowTextWrap":
				  textEditor.WordWrap = !textEditor.WordWrap;

				  if (textEditor.WordWrap)
				    textEditor.HorizontalScrollBarVisibility = ScrollBarVisibility.Hidden;
				  else
				    textEditor.HorizontalScrollBarVisibility = ScrollBarVisibility.Visible;

				  break;

				case "mnuShowLineNumber":
					textEditor.ShowLineNumbers = !textEditor.ShowLineNumbers;
					break;

				case "mnuShowMark":
					textEditor.Options.ShowEndOfLine = !textEditor.Options.ShowEndOfLine;
					break;

				case "mnuShowSpace":
					textEditor.Options.ShowSpaces = !textEditor.Options.ShowSpaces;
					break;

				case "mnuShowTab":
					textEditor.Options.ShowTabs = !textEditor.Options.ShowTabs;
					break;

				case "mnuShowText":
						textEditor.SyntaxHighlighting = FTxtDefinition;
					break;

				case "mnuShowCSharp":
					FCurrentDefinition = TextEditorUtility.GetHighlightingByName("C#");
					if (FCurrentDefinition != null)
						textEditor.SyntaxHighlighting = FCurrentDefinition;
					break;

				case "mnuShowVB":
					FCurrentDefinition = TextEditorUtility.GetHighlightingByName("VBNET");
					if (FCurrentDefinition != null)
						textEditor.SyntaxHighlighting = FCurrentDefinition;
					break;

				case "mnuShowHtml":
					FCurrentDefinition = TextEditorUtility.GetHighlightingByName("HTML");
					if (FCurrentDefinition != null)
						textEditor.SyntaxHighlighting = FCurrentDefinition;
					break;

				case "mnuShowXml":
					FCurrentDefinition = TextEditorUtility.GetHighlightingByName("XML");
					if (FCurrentDefinition != null)
						textEditor.SyntaxHighlighting = FCurrentDefinition;
					break;

				case "mnuShowNoFolding":
					FFoldingStrategy = null;
					break;

				case "mnuShowCSharpFolding":
					FFoldingStrategy = new BraceFoldingStrategy();
					break;

				case "mnuShowXmlFolding":
					FFoldingStrategy = new XmlFoldingStrategy();
					break;

				case "mnuShowFunctionBar":
					if (borderFunction.Visibility == Visibility.Visible)
					{
						borderFunction.Visibility = Visibility.Collapsed;
						mnuShowFunctionBar.IsChecked = false;
					}
					else
					{
						borderFunction.Visibility = Visibility.Visible;
						mnuShowFunctionBar.IsChecked = true;
					}
					break;
			}

			if (FFoldingStrategy != null)
			{
				if (FFoldingManager == null)
				{
					FFoldingManager = FoldingManager.Install(textEditor.TextArea);
				}

				FFoldingStrategy.UpdateFoldings(FFoldingManager, textEditor.Document);
			}
			else
			{
				if (FFoldingManager != null)
				{
					FoldingManager.Uninstall(FFoldingManager);
					FFoldingManager = null;
				}
			}
		}
		#endregion

		#region help
		//---------------------------------------------------------------------------------------------
		private void mnuHelp_Click(object sender, RoutedEventArgs e)
		{
			MenuItem menuItem = sender as MenuItem;

			if (menuItem == null)
				return;

			switch (menuItem.Name)
			{
				case "mnuHelpHelp":
					MessageBox.Show("実装はありません。");
					break;

				case "mnuHelpVersion":
					MessageBox.Show("実装はありません。");
					break;

				case "mnuHelpCustomize":
					var form = new Customize();
					form.OwnerWindow = this;

					if (form.ShowDialog() == true)
					{
						this.InitializeControl();
					}

					break;
			}
		}
		#endregion
		//---------------------------------------------------------------------------------------------
		private void Caret_PositionChanged(object sender, EventArgs e)
		{
			var location = TextEditorUtility.GetCaretLocation(textEditor);
			statusLabel2.Text = String.Format("{0}  行  {1} 列", location.Line, location.Column);
		}

		//---------------------------------------------------------------------------------------------
		private void textEditor_DragEnter(object sender, DragEventArgs e)
		{
			if (e.Data.GetDataPresent(DataFormats.FileDrop))
				e.Effects = DragDropEffects.Copy;
			else
				e.Effects = DragDropEffects.None;
		}

		//---------------------------------------------------------------------------------------------
		private void textEditor_Drop(object sender, DragEventArgs e)
		{
			if (e.Data.GetDataPresent(DataFormats.FileDrop))
			{
				var fileNames = e.Data.GetData(DataFormats.FileDrop, false) as string[];

				if (fileNames != null)
				{
					textEditor.Load(fileNames[0]);
				}
			}
		}

		//---------------------------------------------------------------------------------------------
		private void cmbSearch_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.Key == Key.Return)
			{
				FSearchList.AddNew(cmbSearch.Text);
				cmbSearch.ItemsSource = null;
				cmbSearch.ItemsSource = FSearchList;

				if (FSearchRenderer != null)
				{
					FSearchManager.Close(textEditor, FSearchRenderer);
					FSearchRenderer = null;
				}
			}
		}

		//---------------------------------------------------------------------------------------------
		private void funcButton_Click(object sender, RoutedEventArgs e)
		{
			Button button = sender as Button;

			if (button == null)
				return;

			switch (button.Name)
			{
				case "btnF1":
					System.Windows.Documents.EditingCommands.MoveToDocumentStart.Execute(null, textEditor.TextArea);
					break;
				case "btnF2":
					System.Windows.Documents.EditingCommands.MoveToDocumentEnd.Execute(null, textEditor.TextArea);
					break;
				case "btnF3":
					if (Keyboard.Modifiers == ModifierKeys.Shift)
						this.mnuSearch_Click(mnuSearchPrevious, null);
					else
						this.mnuSearch_Click(mnuSearchNext, null);
					break;
				case "btnF4":
					break;
				case "btnF5":
					break;
				case "btnF6":
					break;
				case "btnF7":
					break;
				case "btnF8":
					break;
				case "btnF9":
					ApplicationCommands.Copy.Execute(null, textEditor.TextArea);
					break;
				case "btnF10":
					ApplicationCommands.Cut.Execute(null, textEditor.TextArea);
					break;
				case "btnF11":
					ApplicationCommands.Paste.Execute(null, textEditor.TextArea);
					break;
				case "btnF12":
					textEditor.SelectAll();
					break;
			}

			textEditor.Focus();
		}

		//---------------------------------------------------------------------------------------------
		private string GetSelectedItemName(object sender)
		{
			string name = String.Empty;
			MenuItem menuItem = sender as MenuItem;

			if (menuItem == null)
			{
				Button button = sender as Button;

				if (button != null)
					name = button.Name;
			}
			else
				name = menuItem.Name;

			return name;
		}

		//---------------------------------------------------------------------------------------------
		// 初期化ファイルからデータを読み込んで検索文字列をコンボボックスに設定する
		private void SetupSearchComboBox()
		{
			// 簡易検索用リスト
			FSearchList = new emanual.Wpf.Utility.HistoryList(cmbSearch, 10);

			var xmlIni = new emanual.Wpf.Utility.XmlIni(g.INI_FILE);
			List<string> list = xmlIni.ReadKeys(g.SIMPLE_SEARCH_SEC);

			for (int i = 0; i < list.Count; ++i)
			{
				string key = list[i];

				string s = (string)xmlIni.ReadValue(g.SIMPLE_SEARCH_SEC, key, "");
				FSearchList.Add(s);
			}

			if (FSearchList.Count > 0)
			{
				cmbSearch.ItemsSource = FSearchList;
				cmbSearch.SelectedIndex = 0;
			}
		}

		//---------------------------------------------------------------------------------------------
		// 検索文字列のリストを初期化ファイルに出力する
		private void WriteSearchComboBox()
		{
			var xmlIni = new emanual.Wpf.Utility.XmlIni(g.INI_FILE);

			for (int i = 0; i < FSearchList.Count; ++i)
			{
				string key = String.Format("Item-{0}", i);

				xmlIni.WriteValue(g.SIMPLE_SEARCH_SEC, key, FSearchList[i]);
			}
		}

		//---------------------------------------------------------------------------------------------
		// 初期化ファイルからデータを読み込んで、TextEditor コントロールを（再）初期化する
		private void InitializeControl()
		{
			var Ini = new emanual.Wpf.Utility.XmlIni(g.INI_FILE);
			string text = textEditor.Document.Text; // 現在のテキストを一時的に保存

			textEditor.BeginChange();
			textEditor.Document.Text = String.Empty; // テキストを削除

			textEditor.FontFamily = (FontFamily)Ini.ReadValue(g.GENERAL_SEC, g.FONTFAMILY_KEY, new FontFamily("Meiryo UI"));
			textEditor.FontSize = (double)Ini.ReadValue(g.GENERAL_SEC, g.FONTSIZE_KEY, 13.0);
			textEditor.Options.NewLine = (string)Ini.ReadValue(g.GENERAL_SEC, g.NEWLINE_KEY, "\u21B5");
			textEditor.Options.IndentationSize = (int)Ini.ReadValue(g.GENERAL_SEC, g.TABSIZE_KEY, 2);

			textEditor.Background = (SolidColorBrush)Ini.ReadValue(g.BRUSH_SEC, g.BACKGROUND_KEY, SystemColors.WindowBrush);
			textEditor.Foreground = (SolidColorBrush)Ini.ReadValue(g.BRUSH_SEC, g.FOREGROUND_KEY, SystemColors.WindowTextBrush);
			textEditor.TextArea.Caret.CaretBrush = (SolidColorBrush)Ini.ReadValue(g.BRUSH_SEC, g.CARET_KEY, SystemColors.WindowTextBrush);
			textEditor.TextArea.TextView.LinkTextForegroundBrush = (SolidColorBrush)Ini.ReadValue(g.BRUSH_SEC, g.HYPERLINK_KEY, Brushes.Blue);
			textEditor.TextArea.TextView.NonPrintableCharacterBrush = (SolidColorBrush)Ini.ReadValue(g.BRUSH_SEC, g.NEWLINE_KEY, Brushes.DimGray);
			textEditor.LineNumbersForeground = (SolidColorBrush)Ini.ReadValue(g.BRUSH_SEC, g.LINENUMBER_KEY, Brushes.DimGray);
			textEditor.Options.SearchMarkerBrush = (SolidColorBrush)Ini.ReadValue(g.BRUSH_SEC, g.SEARCH_KEY, Brushes.Orange);

			textEditor.Document.Text = text; // 元のテキストを復元（こうしないとコントロールの再描画時にエラーになる）
			textEditor.EndChange();
		}

		//---------------------------------------------------------------------------------------------
	} // end of MainWindow class
} // end of namespace
