﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using ICSharpCode.AvalonEdit.Search;
using System.Diagnostics;

namespace emanual.Wpf.TextEditor
{
	public partial class TextSearch : Window
	{
		private emanual.Wpf.Utility.HistoryList FHistoryList;
		private SearchManager FSearchManager = null;
		private SearchResultBackgroundRenderer FSearchRenderer = null;
		private ICSharpCode.AvalonEdit.TextEditor FTextEditor = null;

		private Window FOwnerWindow = null;
		private string FSearchString = String.Empty;
		private bool FIgnoreCase = true;
		private bool FIsReplaceAll = false; // すべて置換のとき true
		private bool FIsWholeWord = false;  // 単語の検索
		private SearchMode FSearchMode = SearchMode.Normal; // 検索の方法

		public Window OwnerWindow { set { FOwnerWindow = value; }}
		public string SearchString { get { return FSearchString; } set { FSearchString = value; } }
		public bool IgnoreCase { get { return FIgnoreCase; } set { FIgnoreCase = value; } }
		public bool IsReplaceAll { get { return FIsReplaceAll; } set { FIsReplaceAll = value; } }
		public bool IsWholeWord { get { return FIsWholeWord; } set { FIsWholeWord = value; } }
		public SearchMode SearchMode { get { return FSearchMode; } set { FSearchMode = value; } }

		public SearchManager SearchManager { get { return FSearchManager; } }
		public SearchResultBackgroundRenderer SearchRenderer { get { return FSearchRenderer; } }

		//---------------------------------------------------------------------------------------------
		// コンストラクタ
		public TextSearch(Window ownerWindow, ICSharpCode.AvalonEdit.TextEditor textEditor)
		{
			InitializeComponent();
			FOwnerWindow = ownerWindow;
			FTextEditor = textEditor;
		}

		//---------------------------------------------------------------------------------------------
		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			if (FOwnerWindow != null)
			{
				this.Left = FOwnerWindow.Left + FOwnerWindow.ActualWidth - this.Width;
				this.Top = FOwnerWindow.Top + 50;
			}
			else
				this.WindowStartupLocation = WindowStartupLocation.CenterOwner; // このケースは事実上ないはずだが

			FSearchManager = new emanual.Wpf.TextEditor.SearchManager(FTextEditor, FSearchRenderer);

			cmbSearch.Text = FSearchString;

			FHistoryList = new emanual.Wpf.Utility.HistoryList(cmbSearch, 10);

			// 初期化ファイルから過去の項目を読み込み、cmbSearch に設定する
			this.SetupSearchComboBox();

			Keyboard.Focus(cmbSearch);
		}

		//---------------------------------------------------------------------------------------------
		private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			if (FSearchRenderer != null)
			{
				FSearchManager.Close(FTextEditor, FSearchRenderer);
			}

			// cmbSearchString の内容を初期化ファイルに出力する
			this.WriteSearchComboBox();
		}

		//---------------------------------------------------------------------------------------------
		private void Window_PreviewKeyDown(object sender, System.Windows.Input.KeyEventArgs e)
		{
			if (e.Key == Key.Escape)
			{
				this.DialogResult = false;
			}
		}

		//---------------------------------------------------------------------------------------------
		private void button_Click(object sender, RoutedEventArgs e)
		{
			Button button = sender as Button;

			if (button == null)
				return;

			switch (button.Name)
			{
				case "btnSearch":
					FSearchString = cmbSearch.Text;
					FHistoryList.AddNew(FSearchString);
					this.DialogResult = true;
					break;

				case "btnCancel":
					this.DialogResult = false;
					break;

				case "btnHelp":
					break;
			}
		}

		//---------------------------------------------------------------------------------------------
		private void checkBox_Checked(object sender, RoutedEventArgs e)
		{
			CheckBox checkBox = sender as CheckBox;

			if (checkBox == null)
				return;

			switch (checkBox.Name)
			{
				case "chkCase":
					FIgnoreCase = (bool)chkCase.IsChecked ? false : true;
					break;
				case "chkWholeWord":
					FIsWholeWord = (bool)chkWholeWord.IsChecked ? true : false;
					break;
				case "chkRegex":
					FSearchMode = (bool)chkRegex.IsChecked ? SearchMode.RegEx : SearchMode.Normal;
					break;
			}
		}

		//---------------------------------------------------------------------------------------------
		// 初期化ファイルからデータを読み込んで検索文字列をコンボボックスに設定する
		private void SetupSearchComboBox()
		{
			var xmlIni = new emanual.Wpf.Utility.XmlIni(g.INI_FILE);
			List<string> list = xmlIni.ReadKeys(g.SEARCH_SEC);

			for (int i = 0; i < list.Count; ++i)
			{
				string key = list[i];

				string s = (string)xmlIni.ReadValue(g.SEARCH_SEC, key, "");
				FHistoryList.Add(s);
			}

			if (FHistoryList.Count > 0)
				cmbSearch.ItemsSource = FHistoryList;
		}

		//---------------------------------------------------------------------------------------------
		// 検索文字列のリストを初期化ファイルに出力する
		private void WriteSearchComboBox()
		{
			var xmlIni = new emanual.Wpf.Utility.XmlIni(g.INI_FILE);

			for (int i = 0; i < FHistoryList.Count; ++i)
			{
				string key = String.Format("Item-{0}", i);

				xmlIni.WriteValue(g.SEARCH_SEC, key, FHistoryList[i]);
			}
		}

		//---------------------------------------------------------------------------------------------
	} // end of TextSearch class
} // end of namespace
