﻿// Copyright (c) AlphaSierraPapa for the SharpDevelop Team (for details please see \doc\copyright.txt)
// This code is distributed under the GNU LGPL (for details please see \doc\license.txt)

using System;
using ICSharpCode.AvalonEdit.Document;

namespace ICSharpCode.AvalonEdit.Editing
{
	// Represents a selected segment.
	public class SelectionSegment : ISegment
	{
		readonly int startOffset, endOffset;
		readonly int startVC, endVC;

		// Creates a SelectionSegment from two offsets.
		public SelectionSegment(int startOffset, int endOffset)
		{
			this.startOffset = Math.Min(startOffset, endOffset);
			this.endOffset = Math.Max(startOffset, endOffset);
			this.startVC = this.endVC = -1;
		}

		// Creates a SelectionSegment from two offsets and visual columns.
		public SelectionSegment(int startOffset, int startVC, int endOffset, int endVC)
		{
			if (startOffset < endOffset || (startOffset == endOffset && startVC <= endVC))
			{
				this.startOffset = startOffset;
				this.startVC = startVC;
				this.endOffset = endOffset;
				this.endVC = endVC;
			}
			else
			{
				this.startOffset = endOffset;
				this.startVC = endVC;
				this.endOffset = startOffset;
				this.endVC = startVC;
			}
		}

		// Gets the start offset.
		public int StartOffset
		{
			get { return startOffset; }
		}

		// Gets the end offset.
		public int EndOffset
		{
			get { return endOffset; }
		}

		// Gets the start visual column.
		public int StartVisualColumn
		{
			get { return startVC; }
		}

		// Gets the end visual column.
		public int EndVisualColumn
		{
			get { return endVC; }
		}

		int ISegment.Offset
		{
			get { return startOffset; }
		}

		public int Length
		{
			get { return endOffset - startOffset; }
		}

		public override string ToString()
		{
			return string.Format("[SelectionSegment StartOffset={0}, EndOffset={1}, StartVC={2}, EndVC={3}]", startOffset, endOffset, startVC, endVC);
		}
	}
}
