﻿// Copyright (c) AlphaSierraPapa for the SharpDevelop Team (for details please see \doc\copyright.txt)
// This code is distributed under the GNU LGPL (for details please see \doc\license.txt)

using System;
using ICSharpCode.AvalonEdit.Document;
using ICSharpCode.AvalonEdit.Utils;

namespace ICSharpCode.AvalonEdit.Highlighting
{
	// Represents a highlighted document.
	// remarks : This interface is used by the "HighlightingColorizer" to register the highlighter as a TextView service.
	public interface IHighlighter
	{
		// Gets the underlying text document.
		TextDocument Document { get; }

		// Gets the span stack at the end of the specified line.
		// -> GetSpanStack(1) returns the spans at the start of the second line.
		// remarks : GetSpanStack(0) is valid and will always return the empty stack.
		ImmutableStack<HighlightingSpan> GetSpanStack(int lineNumber);

		// Highlights the specified document line.
		// lineNumber : The line to highlight.
		// returns : A "HighlightedLine" line object that represents the highlighted sections.
		HighlightedLine HighlightLine(int lineNumber);
	}
}
