﻿// Copyright (c) AlphaSierraPapa for the SharpDevelop Team (for details please see \doc\copyright.txt)
// This code is distributed under the GNU LGPL (for details please see \doc\license.txt)

using ICSharpCode.AvalonEdit.Utils;
using System;
using ICSharpCode.AvalonEdit.Document;

namespace ICSharpCode.AvalonEdit.Indentation
{
	// Handles indentation by copying the indentation from the previous line.
	// Does not support indenting multiple lines.
	public class DefaultIndentationStrategy : IIndentationStrategy
	{
		public virtual void IndentLine(TextDocument document, DocumentLine line)
		{
			if (document == null)
				throw new ArgumentNullException("document");
			if (line == null)
				throw new ArgumentNullException("line");

			DocumentLine previousLine = line.PreviousLine;

			if (previousLine != null)
			{
				ISegment indentationSegment = TextUtilities.GetWhitespaceAfter(document, previousLine.Offset);
				string indentation = document.GetText(indentationSegment);
				// copy indentation to line
				indentationSegment = TextUtilities.GetWhitespaceAfter(document, line.Offset);
				document.Replace(indentationSegment, indentation);
			}
		}

		// Does nothing: indenting multiple lines is useless without a smart indentation strategy.
		public virtual void IndentLines(TextDocument document, int beginLine, int endLine)
		{
		}
	}
}
