﻿// Copyright (c) AlphaSierraPapa for the SharpDevelop Team (for details please see \doc\copyright.txt)
// This code is distributed under the GNU LGPL (for details please see \doc\license.txt)

using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

using ICSharpCode.AvalonEdit.Document;
using ICSharpCode.AvalonEdit.Editing;
using ICSharpCode.AvalonEdit.Rendering;

namespace ICSharpCode.AvalonEdit.Search
{
	public class SearchResultBackgroundRenderer : IBackgroundRenderer //<< internal を public に変更（MainWindow で使うため）
	{
		private TextSegmentCollection<SearchResult> FCurrentResults = new TextSegmentCollection<SearchResult>();
		private Brush FMarkerBrush = Brushes.LightGreen; //<<
		private Pen FMarkerPen = new Pen(Brushes.LightGreen, 1); //<<

		public TextSegmentCollection<SearchResult> CurrentResults
		{
			get { return FCurrentResults; }
		}

		public KnownLayer Layer
		{
			get
			{
				// draw behind selection
				return KnownLayer.Selection;
			}
		}

		public Brush MarkerBrush //<< 独自に追加
		{
			get { return FMarkerBrush; }
			set
			{
				this.FMarkerBrush = value;
				FMarkerPen = new Pen(FMarkerBrush, 1);
			}
		}
		//---------------------------------------------------------------------------------------------
		public SearchResultBackgroundRenderer()
		{
		}

		//---------------------------------------------------------------------------------------------
		public void Draw(TextView textView, DrawingContext drawingContext)
		{
			if (textView == null)
				throw new ArgumentNullException("textView");
			if (drawingContext == null)
				throw new ArgumentNullException("drawingContext");

			if (FCurrentResults == null || !textView.VisualLinesValid)
				return;

			var visualLines = textView.VisualLines;
			if (visualLines.Count == 0)
				return;

			double cornerRadius = textView.Options.MarkerCornerRadius; //<<
			int viewStart = visualLines.First().FirstDocumentLine.Offset;
			int viewEnd = visualLines.Last().LastDocumentLine.EndOffset;

			foreach (SearchResult result in FCurrentResults.FindOverlappingSegments(viewStart, viewEnd - viewStart))
			{
				var geoBuilder = new BackgroundGeometryBuilder();
				geoBuilder.AlignToMiddleOfPixels = true;
				geoBuilder.CornerRadius = cornerRadius; //<< オリジナルは 3 で固定だが、変数化した
				geoBuilder.AddSegment(textView, result);
				Geometry geometry = geoBuilder.CreateGeometry();

				if (geometry != null)
				{
					drawingContext.DrawGeometry(FMarkerBrush, FMarkerPen, geometry); //<<
				}
			}
		}
	}
}
