﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace emanual.Wpf.Controls
{
	public partial class ColorPicker : Window
	{
		public Window OwnerWindow = null;

		public static readonly DependencyProperty SelectedColorProperty;
		public static readonly RoutedEvent SelectedColorChangedEvent;

		public Color SelectedColor
		{
			get { return (Color)this.GetValue(SelectedColorProperty); }
			set { this.SetValue(SelectedColorProperty, value); }
		}

		// SelectedColor プロパティが変化したときに発生するイベントを設定・取得する
		public event RoutedPropertyChangedEventHandler<Color> SelectedColorChanged
		{
			add { this.AddHandler(SelectedColorChangedEvent, value); }
			remove { this.RemoveHandler(SelectedColorChangedEvent, value); }
		}

		
		//---------------------------------------------------------------------------------------------
		static ColorPicker()
		{
			SelectedColorProperty = DependencyProperty.Register("SelectedColor", typeof(Color), typeof(ColorPicker),
				new FrameworkPropertyMetadata(Colors.White, new PropertyChangedCallback(OnSelectedColorChanged)));

			SelectedColorChangedEvent = EventManager.RegisterRoutedEvent("SelectedColorChanged",
				RoutingStrategy.Bubble, typeof(RoutedPropertyChangedEventHandler<Color>), typeof(ColorPicker));
		}

		//---------------------------------------------------------------------------------------------
		public ColorPicker()
		{
			InitializeComponent();
		}

		//---------------------------------------------------------------------------------------------
		private void Window_Loaded(object sender, RoutedEventArgs e)
		{
			if (this.OwnerWindow != null)
			{
				this.Left = this.OwnerWindow.Left + 100;
				this.Top = this.OwnerWindow.Top + 50;
			}
			else
				this.WindowStartupLocation = WindowStartupLocation.CenterOwner;
		}

		//---------------------------------------------------------------------------------------------
		private static void OnSelectedColorChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
		{
			ColorPicker control = (ColorPicker)obj;

			RoutedPropertyChangedEventArgs<Color> e = new RoutedPropertyChangedEventArgs<Color>(
					(Color)args.OldValue, (Color)args.NewValue, SelectedColorChangedEvent);

			control.OnSelectedColorChanged(e);
		}

		//---------------------------------------------------------------------------------------------
		protected virtual void OnSelectedColorChanged(RoutedPropertyChangedEventArgs<Color> args)
		{
			this.RaiseEvent(args);
		}

		//---------------------------------------------------------------------------------------------
		private void ColorListBox_SelectedColorChanged(object sender, RoutedPropertyChangedEventArgs<Color> e)
		{
			this.SelectedColor = colorListBox.SelectedColor;
			borderColor.Background = new SolidColorBrush(this.SelectedColor);
			txtColor.Foreground = this.GetTextColor((SolidColorBrush)borderColor.Background);
			txtColor.Text = this.SelectedColor.ToString();
		}

		//---------------------------------------------------------------------------------------------
		private void btnOK_Click(object sender, RoutedEventArgs e)
		{
			this.DialogResult = true;
		}

		//---------------------------------------------------------------------------------------------
		private void btnCancel_Click(object sender, RoutedEventArgs e)
		{
			this.DialogResult = false;
		}

		//---------------------------------------------------------------------------------------------
		// 背景色の明るさに応じて文字色（黒または白）を返す
		private Brush GetTextColor(SolidColorBrush brush)
		{
			Color color = brush.Color;
			Brush textBrush = Brushes.White;
			double grayScale = 0.299 * color.R + 0.587 * color.G + 0.114 * color.B;

			// 128 より明るいとき
			if (grayScale > 128)
				textBrush = Brushes.Black;

			return textBrush;
		}

		//---------------------------------------------------------------------------------------------
	} // end of ColorPicker class
}
