﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Controls.Primitives;

namespace emanual.Wpf.TextEditor
{
	public class ColorPopup : Popup
	{
		private List<ColorData> FList;

		private Color FSelectedColor;
		private string FColorName;

		public Color SelectedColor { get { return FSelectedColor; } set { FSelectedColor = value; } }
		public string ColorName { get { return FColorName; } set { FColorName = value; } }

		//---------------------------------------------------------------------------------------------
		public ColorPopup()
		{
			this.SetupList();

			this.AllowsTransparency = true;
			this.Placement = PlacementMode.Right;
			this.PopupAnimation = PopupAnimation.Scroll;

			var wrapPanel = new WrapPanel();
			wrapPanel.Orientation = Orientation.Horizontal;
			wrapPanel.VerticalAlignment = VerticalAlignment.Top;
			wrapPanel.HorizontalAlignment = HorizontalAlignment.Left;
			wrapPanel.ItemWidth = 26;
			wrapPanel.ItemHeight = 26;
			wrapPanel.Width = 140;
			wrapPanel.Background = Brushes.Wheat;

			for (int i = 0; i < FList.Count; ++i)
			{
				var button = new Button();
				button.Width = 20;
				button.Height = 20;
				button.ToolTip = FList[i].Name;
				button.Tag = FList[i];
				button.Style = Application.Current.FindResource("STYLE_ImageButton") as Style;
				button.Click += new RoutedEventHandler(button_Click);

				var border = new Border();
				border.Width = 16;
				border.Height = 16;
				border.Background = new SolidColorBrush(FList[i].Color);
				border.BorderBrush = Brushes.Gray;
				border.BorderThickness = new Thickness(1);
				border.HorizontalAlignment = HorizontalAlignment.Center;
				border.VerticalAlignment = VerticalAlignment.Center;

				button.Content = border;

				wrapPanel.Children.Add(button);
			}

			var outerBorder = new Border();
			outerBorder.BorderBrush = Brushes.Orange;
			outerBorder.BorderThickness = new Thickness(2);
			outerBorder.Child = wrapPanel;

			this.Child = outerBorder;
			this.StaysOpen = false;
		}

		//---------------------------------------------------------------------------------------------
		protected void button_Click(object sender, RoutedEventArgs e)
		{
			Button button = sender as Button;

			if (button != null)
			{
				ColorData data = (ColorData)button.Tag;
				FSelectedColor = data.Color;
				FColorName = data.Name;
				this.IsOpen = false;
			}
		}

		//---------------------------------------------------------------------------------------------
		protected override void OnPreviewKeyDown(System.Windows.Input.KeyEventArgs e)
		{
			base.OnPreviewKeyDown(e);

			if (e.Key == System.Windows.Input.Key.Escape)
				this.IsOpen = false;
		}

		//---------------------------------------------------------------------------------------------
		private void SetupList()
		{
			FList = new List<ColorData>();
			FList.Add(new ColorData(Colors.Black, "Black"));
			FList.Add(new ColorData(Colors.DimGray, "DimGray"));
			FList.Add(new ColorData(Colors.Gray, "Gray"));
			FList.Add(new ColorData(Colors.Silver, "Silver"));
			FList.Add(new ColorData(Colors.White, "White"));

			FList.Add(new ColorData(Colors.Maroon, "Maroon"));
			FList.Add(new ColorData(Colors.Red, "Red"));
			FList.Add(new ColorData(Colors.Orange, "Orange"));
			FList.Add(new ColorData(Colors.Yellow, "Yellow"));
			FList.Add(new ColorData(Colors.Fuchsia, "Fuchsia"));

			FList.Add(new ColorData(Colors.DarkGreen, "DarkGreen"));
			FList.Add(new ColorData(Colors.Green, "Green"));
			FList.Add(new ColorData(Colors.Teal, "Teal"));
			FList.Add(new ColorData(Colors.Lime, "Lime"));
			FList.Add(new ColorData(Colors.Olive, "Olive"));

			FList.Add(new ColorData(Colors.Navy, "Navy"));
			FList.Add(new ColorData(Colors.Blue, "Blue"));
			FList.Add(new ColorData(Colors.DodgerBlue, "DodgerBlue"));
			FList.Add(new ColorData(Colors.Cyan, "Cyan"));
			FList.Add(new ColorData(Colors.Purple, "Purple"));
		}
	} // end of ColorPopup class

	//************************************************************************************************
	internal class ColorData
	{
		public Color Color { get; set; }
		public string Name { get; set; } // 色名（例："Purple"）

		public ColorData(Color color, string name)
		{
			this.Color = color;
			this.Name = name;
		}
	}
} // end of namespace
